@@ -165,13 +165,15 @@ webkit_pref_get_internal_user_agent (void)
   initialized = TRUE;
 
   vendor_user_agent = webkit_pref_get_vendor_user_agent ();
-  if (!vendor_user_agent)
-    return NULL;
-
   webkit_user_agent = webkit_settings_get_user_agent (webkit_settings);
-  user_agent = g_strdup_printf ("%s %s Epiphany/%s", webkit_user_agent,
-                                vendor_user_agent, VERSION);
-  g_free (vendor_user_agent);
+
+  if (vendor_user_agent) {
+    user_agent = g_strdup_printf ("%s %s Epiphany/%s", webkit_user_agent,
+                                  vendor_user_agent, VERSION);
+    g_free (vendor_user_agent);
+  } else {
+    user_agent = g_strdup_printf ("%s Epiphany/%s", webkit_user_agent, VERSION);
+  }
 
   return user_agent;
 }
@@ -181,23 +183,26 @@ webkit_pref_callback_user_agent (GSettings  *settings,
                                  const char *key,
                                  gpointer    data)
 {
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   char *value;
-  const char *internal_user_agent;
+  char *user_agent;
+  const char *base_user_agent;
 
   value = g_settings_get_string (settings, key);
-  if (value != NULL && value[0] != '\0') {
-    webkit_settings_set_user_agent (webkit_settings, value);
-    g_free (value);
-    return;
-  }
-  g_free (value);
-
-  internal_user_agent = webkit_pref_get_internal_user_agent ();
-  if (internal_user_agent)
-    webkit_settings_set_user_agent (webkit_settings, internal_user_agent);
+  if (value != NULL && value[0] != '\0')
+    base_user_agent = value;
   else
-    webkit_settings_set_user_agent_with_application_details (webkit_settings,
-                                                             "Epiphany", VERSION);
+    base_user_agent = webkit_pref_get_internal_user_agent ();
+
+  if (ephy_embed_shell_get_mode (shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
+    user_agent = g_strdup_printf ("%s (Web App)", base_user_agent);
+  else
+    user_agent = g_strdup (base_user_agent);
+
+  webkit_settings_set_user_agent (webkit_settings, user_agent);
+
+  g_free (user_agent);
+  g_free (value);
 }
 
 static gdouble
